﻿/**************************************
 * waveファイルを読み込んで、順次FFTを実行・描画するサンプルプログラム
 * [開発者]     森下功啓(Katsuhiro Morishita)
 * 
 * [参考文献]   オブジェクト配列について，http://www.apfield.jp/csgarden/checkbox.html
 * 
 * [メモ]
 *  おまけとしてオブジェクトを配列で宣言して制御するコードのテストを兼ねている。
 *  このコードを見本とすれば、テキストボックスの動的配置は容易い
 *  
 * [履歴]       2011/3     開発を始めたのは確かこの時期だったと思う
 *              2011/10/7  waveクラスの更新に合わせてコードを更新
 * *********************************************/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Signal_Process;                                   // FFTに必要
using System.IO;                                        // ファイル操作に必要
using WAVE_file;                                        // waveファイルの操作に必要

namespace 鳥の声で種識別
{
    public partial class FFTtestForm : Form
    {
        // グローバル変数を宣言
        private const int FFT_arr_size = 5;
        private wave wave_file = new wave();
        private CheckBox[] fft_chbox = new CheckBox[FFT_arr_size];                      // チェックボックスオブジェクトを配列で宣言
        private int[] fft_points = new int[FFT_arr_size] { 256, 512, 1024, 2048, 4096 };// 実施するFFTのサイズを指定する

        public FFTtestForm()
        {
            InitializeComponent();
        }

        /*------------------------------------------
         * FFT実行ボタンが押された時の動作
         *  タイマーを動かしたり、停止させる
         * --------------------------------------------*/
        private void FFT_button_Click(object sender, EventArgs e)
        {
            if (this.FFTtimer.Enabled == true) {
                this.FFTtimer.Enabled = false;
                this.TimerStatusLabel.Text = "タイマー停止中";
            }else if (this.FFTtimer.Enabled == false){
                this.FFTtimer.Enabled = true;
                this.TimerStatusLabel.Text = "タイマー起動中";
            }
        }

        /*------------------------------------------
         * waveファイル選択ボタンが押された時の動作
         *  ファイルを選択し、可能なら開く
         * --------------------------------------------*/
        private void wave_set_button_Click(object sender, EventArgs e)
        {
            openFileDialog1.FileName = "hoge.wave";                                 // ファイルを開く前に初期設定
            openFileDialog1.Filter = "Wave Files (*.wav, *.wave)|*.wav;*.wave";
            if (openFileDialog1.ShowDialog() == DialogResult.OK) {
                this.wave_file.Open(openFileDialog1);
            }
        }
        /*------------------------------------------
         * タイマーで定期的に実行される
         * 
         * --------------------------------------------*/
        private void FFTtimer_Tick(object sender, EventArgs e)
        {
            Graphics g = pictureBox1.CreateGraphics();                              // 図の描画の準備
            Pen wave_kpen = new Pen(Color.Black, 1);
            Pen fft_pen;
            int x1, x2, y1, y2;
            DFT[] fft_arr = new DFT[FFT_arr_size];                                  // FFTの準備
            const double read_size_sec = 0.1;                                       // waveファイルからの読み込み量[s]を指定
            double[] sound;
            int i, k;
            wave.MusicData data;                                                    // 音声ファイルから読み込んだデータが格納される。


            if (this.wave_file.IsOpen && this.wave_file.ReadLimit == false)         // 読めるなら、読み込む
            {
                // read
                data = this.wave_file.Read(read_size_sec);                          // クラスメソッドを使用してファイルからread_size_sec秒間読み込み。実行の度に更新される。
                if (data.UsableCH == wave.MusicData.UsableChannel.Left)              // データの入っている方を格納する
                    sound = data.GetData(wave.MusicData.Channel.Left);               // 
                else
                    sound = data.GetData(wave.MusicData.Channel.Right);

                // wave data plot
                pictureBox1.Refresh();
                g.DrawLine(wave_kpen, 0, pictureBox1.Height / 2, pictureBox1.Width, pictureBox1.Height / 2); // 軸を書く
                x2 = y2 = 0;
                for (i = 0; i < sound.Length - 1; i += (sound.Length / 1000 + 1))                            // 読み込んだデータを描画
                {
                    x1 = x2;
                    y1 = y2;
                    x2 = (i + 1) * pictureBox1.Width / sound.Length;
                    y2 = (int)(-(pictureBox1.Height / 2) * sound[i + 1] / (2 << this.wave_file.Resolution - 1) + pictureBox1.Height / 2);
                    g.DrawLine(wave_kpen, x1, y1, x2, y2);
                }

                // FFT
                for (i = 0; i < FFT_arr_size; i++)
                {
                    fft_arr[i] = new DFT(fft_points[i], DFT.Window.Perzen);                         // インスタンスを生成
                    fft_arr[i].FFT(sound);                                                          // FFTを実行. FFTサイズが音声データよりも小さい場合、始めの方しか処理しないので注意.
                    // plot FFT result
                    if (this.fft_chbox[i].Checked)
                    {
                        fft_pen = new Pen(Color.FromArgb(40 * i, 250 - 50 * i, i * 50), 1);          // カラーの決定
                        x2 = 0;
                        y2 = (int)(-(pictureBox1.Height / 2) * fft_arr[i].calced_datas[0].Log / 5 + pictureBox1.Height);
                        for (k = 1; k < fft_arr[i].FFTsize / 2 - 1; k += ((fft_arr[i].FFTsize / 2) / 1000 + 1))// 描画スピードを優先して、最大1000ポイントの描画とする
                        {
                            x1 = x2;
                            y1 = y2;
                            x2 = k * pictureBox1.Width * 2 / fft_arr[i].FFTsize;                    // FFT実部を描画。
                            y2 = (int)(-(pictureBox1.Height / 2) * fft_arr[i].calced_datas[k].Log / 5 + pictureBox1.Height);
                            if (y1 >= 0 && y2 >= 0) g.DrawLine(fft_pen, x1, y1, x2, y2);
                        }
                    }
                }
                this.TimeAtReadLabel.Text = "読み出し時刻 = " + this.wave_file.NowTime + "[s]";
            }
            else { this.TimeAtReadLabel.Text = "ファイルの読み出しに失敗しました。フォーマットを確認してください。"; }
            g.Dispose();
        }
        /*******************************************************
         * フォームがロードされた際に、最初に実行する内容
         * 
         * *****************************************************/
        private void FFTtestForm_Load(object sender, EventArgs e)
        {
            int i;

            // チェックボックスの設定
            for (i = 0; i < FFT_arr_size; i++)
            {
                this.fft_chbox[i] = new CheckBox();                         // チェックボックス生成
                this.fft_chbox[i].Location = new Point( 10, 20 * i);        // フォーム上に20ピクセル間隔で配置
                this.fft_chbox[i].Width = 80;
                Controls.Add(this.fft_chbox[i]);                            // チェックボックスをコントロールとして追加
                this.fft_chbox[i].Text = "FFT" + this.fft_points[i].ToString();
                this.fft_chbox[i].Tag = Convert.ToString(i);　              // チェックボックス判別用のタグを付ける
                this.fft_chbox[i].BringToFront();                           // 最前面に表示
            }
        }
    }
}
